==> vdr-plugin-webvideo_0.2.1.orig.tar.gz: webvideo-0.2.1/src/vdr-plugin/webvideo.c <==
/*
 * webvideo.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <getopt.h>
#include <time.h>
#include <vdr/plugin.h>
#include <vdr/tools.h>
#include <vdr/videodir.h>
#include <vdr/i18n.h>
#include <vdr/skins.h>
#include <libwebvi.h>
#include "menu.h"
#include "history.h"
#include "download.h"
#include "request.h"
#include "mimetypes.h"
#include "config.h"
#include "player.h"
#include "common.h"

const char *VERSION               = "0.2.0";
static const char *DESCRIPTION    = trNOOP("Download video files from the web");
static const char *MAINMENUENTRY  = "Webvideo";
cMimeTypes *MimeTypes             = NULL;

class cPluginWebvideo : public cPlugin {
private:
  // Add any member variables or functions you may need here.
  cWebviThread *downloadthread;
  cNavigationMenu *navigationmenu;
  cHistory history;

  static int nextMenuID;

  void UpdateOSDFromHistory(const char *statusmsg=NULL);
  void UpdateStatusMenu(bool force=false);
  bool StartStreaming(const cString &streamurl);
public:
  cPluginWebvideo(void);
  virtual ~cPluginWebvideo();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return tr(DESCRIPTION); }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual void MainThreadHook(void);
  virtual cString Active(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  virtual bool Service(const char *Id, void *Data = NULL);
  virtual const char **SVDRPHelpPages(void);
  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
  };

int cPluginWebvideo::nextMenuID = 1;

cPluginWebvideo::cPluginWebvideo(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
  webvideoConfig->SetDownloadPath(VideoDirectory);
}

cPluginWebvideo::~cPluginWebvideo()
{
  // Clean up after yourself!
  webvi_cleanup(0);
}

const char *cPluginWebvideo::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return "  -d DIR,   --downloaddir=DIR  Save downloaded files to DIR\n";
  return "  -t DIR,   --templatedir=DIR  Read video site templates from DIR\n";
}

bool cPluginWebvideo::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  static struct option long_options[] = {
    { "downloaddir", required_argument, NULL, 'd' },
    { "templatedir", required_argument, NULL, 't' },
    { NULL }
  };

  int c;
  while ((c = getopt_long(argc, argv, "d:t:", long_options, NULL)) != -1) {
    switch (c) {
    case 'd':
      webvideoConfig->SetDownloadPath(optarg);
      break;
    case 't':
      webvideoConfig->SetTemplatePath(optarg);
      break;
    default:
      return false;
    }
  }
  return true;
}

bool cPluginWebvideo::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.

  // Test that run-time and compile-time libxml versions are compatible
  LIBXML_TEST_VERSION;

  cString mymimetypes = AddDirectory(ConfigDirectory(Name()), "mime.types");
  const char *mimefiles [] = {"/etc/mime.types", (const char *)mymimetypes, NULL};
  MimeTypes = new cMimeTypes(mimefiles);

  if (webvi_global_init() != 0) {
    error("Failed to initialize libwebvi");
    return false;
  }

  downloadthread = new cWebviThread(webvideoConfig->GetTemplatePath());

  return true;
}

bool cPluginWebvideo::Start(void)
{
  // Start any background activities the plugin shall perform.

  downloadthread->Start();
  return true;
}

void cPluginWebvideo::Stop(void)
{
  // Stop any background activities the plugin shall perform.
  delete downloadthread;
  delete MimeTypes;

  xmlCleanupParser();
}

void cPluginWebvideo::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

void cPluginWebvideo::MainThreadHook(void)
{
  // Perform actions in the context of the main program thread.
  // WARNING: Use with great care - see PLUGINS.html!

  bool forceStatusUpdate = false;
  cMenuRequest *req;
  cString streamurl;

  while ((req = downloadthread->GetFinishedRequest())) {
    int cid = -1;
    if (history.Current()) {
      cid = history.Current()->GetID();
    }

    debug("Finished request: %d (current: %d), type = %d, status = %d", 
          req->GetID(), cid, req->GetType(), req->GetStatusCode());

    if (req->Success()) {
      switch (req->GetType()) {
      case REQT_MENU:
	// Only change the menu if the request was launched from the
	// current menu.
	if (req->GetID() == cid) {
	  if (cid == 0) {
	    // Special case: replace the placeholder menu
	    history.Clear();
	  }

          history.TruncateAndAdd(new cHistoryObject(req->GetResponse(), 
                                                    nextMenuID++));
          UpdateOSDFromHistory();
        }
	break;

      case REQT_STREAM:
        streamurl = req->GetResponse();
        if (streamurl[0] == '\0')
          Skins.Message(mtError, tr("Streaming failed: no URL"));
        else if (!StartStreaming(streamurl))
          Skins.Message(mtError, tr("Failed to launch media player"));
	break;

      case REQT_FILE:
        Skins.Message(mtInfo, cString::sprintf(tr("One download completed, %d remains"), downloadthread->GetUnfinishedCount()));
	forceStatusUpdate = true;
	break;

      default:
	break;
      }
    } else { // failed request
      int code = req->GetStatusCode();
      if (code == -2)
	Skins.Message(mtError, tr("Download aborted"));
      else
	Skins.Message(mtError, cString::sprintf(tr("Download failed (error = %d)"), code));

      debug("request failed (%d: %s)", code, (const char*)req->GetStatusPharse());
      forceStatusUpdate = true;
    }

    delete req;
  }

  UpdateStatusMenu(forceStatusUpdate);
}

cString cPluginWebvideo::Active(void)
{
  // Return a message string if shutdown should be postponed
  int c = downloadthread->GetUnfinishedCount();
  if (c > 0)
    return cString::sprintf(tr("%d downloads not finished"), c);
  else
    return NULL;
}

cOsdObject *cPluginWebvideo::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  const char *placeholderMenu = "<wvmenu><title>Webvideo</title></wvmenu>";
  const char *statusmsg = NULL;

  cHistoryObject *hist = history.Home();
  if (!hist) {
    downloadthread->AddRequest(new cMenuRequest(0, "wvt:///?srcurl=mainmenu"));
    history.TruncateAndAdd(new cHistoryObject(placeholderMenu, 0));
    statusmsg = tr("Retrieving...");
  }

  navigationmenu = new cNavigationMenu(&navigationmenu, &history, downloadthread);

  UpdateOSDFromHistory(statusmsg);
  return navigationmenu;
}

cMenuSetupPage *cPluginWebvideo::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return NULL;
}

bool cPluginWebvideo::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  return false;
}

bool cPluginWebvideo::Service(const char *Id, void *Data)
{
  // Handle custom service requests from other plugins
  return false;
}

const char **cPluginWebvideo::SVDRPHelpPages(void)
{
  // Return help text for SVDRP commands this plugin implements
  return NULL;
}

cString cPluginWebvideo::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode)
{
  // Process SVDRP commands this plugin implements
  return NULL;
}

void cPluginWebvideo::UpdateOSDFromHistory(const char *statusmsg) {
  if (navigationmenu) {
    cHistoryObject *hist = history.Current();
    navigationmenu->Populate(hist, statusmsg);
    navigationmenu->Display();
  } else {
    debug("OSD is not ours.");
  }
}

void cPluginWebvideo::UpdateStatusMenu(bool force) {
  if (navigationmenu) {
    cStatusScreen *st = navigationmenu->GetStatusScreen();
    if (st && (force || st->NeedsUpdate())) {
      st->Update();
    }
  }
}

bool cPluginWebvideo::StartStreaming(const cString &streamurl) {
  cMediaPlayer *players[2];

  if (webvideoConfig->GetPreferXineliboutput()) {
    players[0] = new cXineliboutputPlayer();
    players[1] = new cMPlayerPlayer();
  } else {
    players[0] = new cMPlayerPlayer();
    players[1] = new cXineliboutputPlayer();
  }

  bool ret = false;
  for (int i=0; i<2; i++) {
    if (players[i]->Launch(streamurl)) {
      ret = true;
      break;
    }
  }

  for (int i=0; i<2 ; i++) {
    delete players[i];
  }

  return ret;
}

VDRPLUGINCREATOR(cPluginWebvideo); // Don't touch this!

==> vdr-plugin-webvideo_0.2.2.orig.tar.gz: webvideo-0.2.2/src/vdr-plugin/webvideo.c <==
/*
 * webvideo.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <getopt.h>
#include <time.h>
#include <vdr/plugin.h>
#include <vdr/tools.h>
#include <vdr/videodir.h>
#include <vdr/i18n.h>
#include <vdr/skins.h>
#include <libwebvi.h>
#include "menu.h"
#include "history.h"
#include "download.h"
#include "request.h"
#include "mimetypes.h"
#include "config.h"
#include "player.h"
#include "common.h"

const char *VERSION               = "0.2.2";
static const char *DESCRIPTION    = trNOOP("Download video files from the web");
static const char *MAINMENUENTRY  = "Webvideo";
cMimeTypes *MimeTypes             = NULL;

class cPluginWebvideo : public cPlugin {
private:
  // Add any member variables or functions you may need here.
  cWebviThread *downloadthread;
  cNavigationMenu *navigationmenu;
  cHistory history;

  static int nextMenuID;

  void UpdateOSDFromHistory(const char *statusmsg=NULL);
  void UpdateStatusMenu(bool force=false);
  bool StartStreaming(const cString &streamurl);
public:
  cPluginWebvideo(void);
  virtual ~cPluginWebvideo();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return tr(DESCRIPTION); }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual void MainThreadHook(void);
  virtual cString Active(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  virtual bool Service(const char *Id, void *Data = NULL);
  virtual const char **SVDRPHelpPages(void);
  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
  };

int cPluginWebvideo::nextMenuID = 1;

cPluginWebvideo::cPluginWebvideo(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
  webvideoConfig->SetDownloadPath(VideoDirectory);
}

cPluginWebvideo::~cPluginWebvideo()
{
  // Clean up after yourself!
  webvi_cleanup(0);
}

const char *cPluginWebvideo::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return "  -d DIR,   --downloaddir=DIR  Save downloaded files to DIR\n";
  return "  -t DIR,   --templatedir=DIR  Read video site templates from DIR\n";
}

bool cPluginWebvideo::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  static struct option long_options[] = {
    { "downloaddir", required_argument, NULL, 'd' },
    { "templatedir", required_argument, NULL, 't' },
    { NULL }
  };

  int c;
  while ((c = getopt_long(argc, argv, "d:t:", long_options, NULL)) != -1) {
    switch (c) {
    case 'd':
      webvideoConfig->SetDownloadPath(optarg);
      break;
    case 't':
      webvideoConfig->SetTemplatePath(optarg);
      break;
    default:
      return false;
    }
  }
  return true;
}

bool cPluginWebvideo::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.

  // Test that run-time and compile-time libxml versions are compatible
  LIBXML_TEST_VERSION;

  cString mymimetypes = AddDirectory(ConfigDirectory(Name()), "mime.types");
  const char *mimefiles [] = {"/etc/mime.types", (const char *)mymimetypes, NULL};
  MimeTypes = new cMimeTypes(mimefiles);

  if (webvi_global_init() != 0) {
    error("Failed to initialize libwebvi");
    return false;
  }

  downloadthread = new cWebviThread(webvideoConfig->GetTemplatePath());

  return true;
}

bool cPluginWebvideo::Start(void)
{
  // Start any background activities the plugin shall perform.

  downloadthread->Start();
  return true;
}

void cPluginWebvideo::Stop(void)
{
  // Stop any background activities the plugin shall perform.
  delete downloadthread;
  delete MimeTypes;

  xmlCleanupParser();
}

void cPluginWebvideo::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

void cPluginWebvideo::MainThreadHook(void)
{
  // Perform actions in the context of the main program thread.
  // WARNING: Use with great care - see PLUGINS.html!

  bool forceStatusUpdate = false;
  cMenuRequest *req;
  cString streamurl;

  while ((req = downloadthread->GetFinishedRequest())) {
    int cid = -1;
    if (history.Current()) {
      cid = history.Current()->GetID();
    }

    debug("Finished request: %d (current: %d), type = %d, status = %d", 
          req->GetID(), cid, req->GetType(), req->GetStatusCode());

    if (req->Success()) {
      switch (req->GetType()) {
      case REQT_MENU:
	// Only change the menu if the request was launched from the
	// current menu.
	if (req->GetID() == cid) {
	  if (cid == 0) {
	    // Special case: replace the placeholder menu
	    history.Clear();
	  }

	  if (history.Current())
	    history.Current()->RememberSelected(navigationmenu->Current());
          history.TruncateAndAdd(new cHistoryObject(req->GetResponse(), 
                                                    nextMenuID++));
          UpdateOSDFromHistory();
        }
	break;

      case REQT_STREAM:
        streamurl = req->GetResponse();
        if (streamurl[0] == '\0')
          Skins.Message(mtError, tr("Streaming failed: no URL"));
        else if (!StartStreaming(streamurl))
          Skins.Message(mtError, tr("Failed to launch media player"));
	break;

      case REQT_FILE:
        Skins.Message(mtInfo, cString::sprintf(tr("One download completed, %d remains"), downloadthread->GetUnfinishedCount()));
	forceStatusUpdate = true;
	break;

      default:
	break;
      }
    } else { // failed request
      int code = req->GetStatusCode();
      if (code == -2)
	Skins.Message(mtError, tr("Download aborted"));
      else
	Skins.Message(mtError, cString::sprintf(tr("Download failed (error = %d)"), code));

      debug("request failed (%d: %s)", code, (const char*)req->GetStatusPharse());
      forceStatusUpdate = true;
    }

    delete req;
  }

  UpdateStatusMenu(forceStatusUpdate);
}

cString cPluginWebvideo::Active(void)
{
  // Return a message string if shutdown should be postponed
  int c = downloadthread->GetUnfinishedCount();
  if (c > 0)
    return cString::sprintf(tr("%d downloads not finished"), c);
  else
    return NULL;
}

cOsdObject *cPluginWebvideo::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  const char *placeholderMenu = "<wvmenu><title>Webvideo</title></wvmenu>";
  const char *statusmsg = NULL;
  struct timespec ts;
  ts.tv_sec = 0;
  ts.tv_nsec = 100*1000*1000; // 100 ms

  navigationmenu = new cNavigationMenu(&navigationmenu, &history, downloadthread);

  cHistoryObject *hist = history.Home();
  if (!hist) {
    downloadthread->AddRequest(new cMenuRequest(0, "wvt:///?srcurl=mainmenu"));
    cHistoryObject *placeholder = new cHistoryObject(placeholderMenu, 0);
    history.TruncateAndAdd(placeholder);

    // The main menu response should come right away. Try to update
    // the menu here without having to wait for the next
    // MainThreadHook call by VDR main loop.
    for (int i=0; i<4; i++) {
      nanosleep(&ts, NULL);
      MainThreadHook();
      if (history.Current() != placeholder) {
        return navigationmenu;
      }
    };

    statusmsg = tr("Retrieving...");
  }

  UpdateOSDFromHistory(statusmsg);
  return navigationmenu;
}

cMenuSetupPage *cPluginWebvideo::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return NULL;
}

bool cPluginWebvideo::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  return false;
}

bool cPluginWebvideo::Service(const char *Id, void *Data)
{
  // Handle custom service requests from other plugins
  return false;
}

const char **cPluginWebvideo::SVDRPHelpPages(void)
{
  // Return help text for SVDRP commands this plugin implements
  return NULL;
}

cString cPluginWebvideo::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode)
{
  // Process SVDRP commands this plugin implements
  return NULL;
}

void cPluginWebvideo::UpdateOSDFromHistory(const char *statusmsg) {
  if (navigationmenu) {
    cHistoryObject *hist = history.Current();
    navigationmenu->Populate(hist, statusmsg);
    navigationmenu->Display();
  } else {
    debug("OSD is not ours.");
  }
}

void cPluginWebvideo::UpdateStatusMenu(bool force) {
  if (navigationmenu) {
    cStatusScreen *st = navigationmenu->GetStatusScreen();
    if (st && (force || st->NeedsUpdate())) {
      st->Update();
    }
  }
}

bool cPluginWebvideo::StartStreaming(const cString &streamurl) {
  cMediaPlayer *players[2];

  if (webvideoConfig->GetPreferXineliboutput()) {
    players[0] = new cXineliboutputPlayer();
    players[1] = new cMPlayerPlayer();
  } else {
    players[0] = new cMPlayerPlayer();
    players[1] = new cXineliboutputPlayer();
  }

  bool ret = false;
  for (int i=0; i<2; i++) {
    if (players[i]->Launch(streamurl)) {
      ret = true;
      break;
    }
  }

  for (int i=0; i<2 ; i++) {
    delete players[i];
  }

  return ret;
}

VDRPLUGINCREATOR(cPluginWebvideo); // Don't touch this!
